from typing import Union, Optional

from pydantic.main import BaseModel


class TokenRequest(BaseModel):
    grant_type: Union[None, str]
    client_id: Union[None, str]
    client_secret: Union[None, str]
    scope: Union[None, str]


class Token(BaseModel):
    access_token: str
    expires_in: int
    token_type: str
    expires_at: int


class UserBase(BaseModel):
    id: str
    first_name: str
    last_name: str
    email: str


class UserRequest(UserBase):
    country: str


class User(UserBase):
    full_name: Union[None, str]
    created_at: Optional[Union[None, str]]
    updated_at: Optional[Union[None, str]]
    location: Union[None, str]


class CompanyBase(BaseModel):
    name: str
    legal_name: str
    tax_number: str


class CompanyRequest(CompanyBase):
    user_id: str
    country: str


class Company(CompanyBase):
    id: str
    charge_tax: Union[None, str]
    address_line_1: Union[None, str]
    address_line_2: Union[None, str]
    city: Union[None, str]
    state: Union[None, str]
    zip: Union[None, str]
    phone: Union[None, str]


class WalletAccount(BaseModel):
    id: str
    active: bool
    created_at: Optional[Union[None, str]]
    updated_at: Optional[Union[None, str]]
    balance: int
    currency: str


class NppDetails(BaseModel):
    pay_id: str
    reference: str
    amount: str
    currency: str


class BpayDetails(BaseModel):
    biller_code: str
    reference: str
    amount: str
    currency: str


class VirtualAccountBase(BaseModel):
    id: str
    wallet_account_id: str


class VirtualAccountRequest(VirtualAccountBase):
    pass


class VirtualAccount(VirtualAccountBase):
    routing_number: int
    account_number: int
    currency: str
    active: bool
    created_at: Optional[Union[None, str]]
    updated_at: Optional[Union[None, str]]


class BankAccountBase(BaseModel):
    bank_name: str
    country: str
    account_name: str
    routing_number: str
    account_number: str
    holder_type: str
    account_type: str


class BankDetails(BankAccountBase):
    direct_debit_authority_status: Optional[str]


class BankAccountRequest(BankAccountBase):
    user_id: str


class BankAccount(BaseModel):
    id: str
    active: bool
    verification_status: str
    currency: str
    created_at: Optional[Union[None, str]]
    updated_at: Optional[Union[None, str]]
    bank: Optional[BankDetails]


class SetDisbursementRequest(BaseModel):
    account_id: str


class Disbursement(BaseModel):
    id: str
    amount: int
    currency: str
    batch_id: Optional[Union[None, int]]
    cuscal_payment_transaction_id: Optional[Union[None, str]]
    created_at: Optional[Union[None, str]]
    updated_at: Optional[Union[None, str]]
    state: str


class Transaction(BaseModel):
    description: str
    type: str
    type_method: str
    state: str
    user_id: str
    user_name: str
    account_id: str
    item_name: str
    dynamic_descriptor: str
    amount: int
    currency: str
    debit_credit: str
    created_at: Optional[Union[None, str]]
    updated_at: Optional[Union[None, str]]


class ItemBase(BaseModel):
    id: str
    name: str
    amount: int


class ItemRequest(ItemBase):
    payment_type: int
    buyer_id: str
    seller_id: str


class Item(ItemBase):
    description: Union[None, str]
    created_at: Optional[Union[None, str]]
    updated_at: Optional[Union[None, str]]
    state: str
    payment_type_id: int
    status: int
    deposit_reference: str
    buyer_name: str
    buyer_country: str
    buyer_email: str
    seller_name: str
    seller_country: str
    seller_email: str
    tds_check_state: Optional[str]
    currency: str


class MakePaymentRequest(BaseModel):
    account_id: str


class RefundPaymentRequest(BaseModel):
    refund_amount: Optional[int]
    refund_message: Optional[str]
    account_id: Optional[str]


class AuthorizePaymentRequest(BaseModel):
    account_id: str
    cvv: Optional[str]


class WebhookBase(BaseModel):
    http_method: str
    url: str
    description: Optional[str]


class WebhookRequest(WebhookBase):
    object_type: str


class WebhookUpdateRequest(WebhookBase):
    pass


class Webhook(WebhookBase):
    uuid: str
    enabled: bool
    created_at: str
    updated_at: str
    object_type: str


class DirectDebitAuthority(BaseModel):
    id: str
    created_at: Optional[Union[None, str]]
    updated_at: Optional[Union[None, str]]
    amount: int
    bank_bsb: str
    debit_user_id: str
    state: str


class DirectDebitAuthorityRequest(BaseModel):
    amount: int
    account_id: str


class WithdrawFundsRequest(BaseModel):
    account_id: str
    amount: int
    custom_descriptor: Optional[Union[None, str]]



from datetime import datetime
from faker import Faker

fake = Faker()


def UserFactory(**kwargs):
    data = dict(
        id=fake.word(),
        full_name=fake.name(),
        first_name=fake.first_name(),
        last_name=fake.last_name(),
        created_at=datetime.now().isoformat(),
        updated_at=datetime.now().isoformat(),
        email=fake.email(),
        mobile="",
        phone=None,
        location="AUS",
    )
    data.update(kwargs)
    return data


def CompanyFactory(**kwargs):
    data = dict(
        id=fake.word(),
        name=fake.word(),
        legal_name=fake.word(),
        tax_number=fake.word(),
        charge_tax=None,
        address_line_1=fake.word(),
        address_line_2=fake.word(),
        city=fake.word(),
        state=fake.word(),
        zip=None,
        country=fake.word(),
        phone=None,
        user_id=fake.word(),
    )
    data.update(kwargs)
    return data


def WalletAccountFactory(**kwargs):
    data = dict(
        id=fake.word(),
        active=fake.boolean(),
        created_at=datetime.now().isoformat(),
        updated_at=datetime.now().isoformat(),
        balance=fake.pyint(),
        currency="AUD",
    )
    data.update(kwargs)
    return data


def NppDetailsFactory(**kwargs):
    data = dict(
        pay_id=fake.email(), reference=fake.word(), amount="$0.00", currency="AUD"
    )
    data.update(kwargs)
    return data


def BpayDetailsFactory(**kwargs):
    data = dict(
        biller_code="111111", reference=fake.word(), amount="$0.00", currency="AUD"
    )
    data.update(kwargs)
    return data


def VirtualAccountFactory(**kwargs):
    data = dict(
        id=fake.word(),
        routing_number=fake.pyint(),
        account_number=fake.pyint(),
        wallet_account_id=fake.word(),
        currency="AUD",
        active=fake.boolean(),
        created_at=datetime.now().isoformat(),
        updated_at=datetime.now().isoformat(),
    )
    data.update(kwargs)
    return data


def BankAccountFactory(**kwargs):
    data = dict(
        id=fake.word(),
        active=fake.boolean(),
        created_at=datetime.now().isoformat(),
        updated_at=datetime.now().isoformat(),
        verification_status="not_verified",
        currency="AUD",
        bank=dict(
            bank_name=fake.word(),
            country="AUS",
            account_name=fake.first_name() + " " + fake.last_name(),
            routing_number="XXXXX1",
            account_number="XXX123",
            holder_type="personal",
            account_type="checking",
            direct_debit_authority_status="approved",
        ),
    )
    data.update(kwargs)
    return data


def DisbursementFactory(**kwargs):
    data = dict(
        id=fake.word(),
        amount=fake.pyint(),
        currency="AUD",
        batch_id=fake.pyint(),
        cuscal_payment_transaction_id=None,
        created_at=datetime.now().isoformat(),
        updated_at=datetime.now().isoformat(),
        state="successful",
    )
    data.update(kwargs)
    return data


def TransactionFactory(**kwargs):
    data = dict(
        description=fake.word(),
        type="payment",
        type_method="credit_card",
        state="successful",
        user_id=fake.word(),
        user_name=fake.first_name() + " " + fake.last_name(),
        account_id=fake.word(),
        item_name=fake.word(),
        dynamic_descriptor=fake.word(),
        amount=fake.pyint(),
        currency="AUD",
        debit_credit="debit",
        created_at=datetime.now().isoformat(),
        updated_at=datetime.now().isoformat(),
    )
    data.update(kwargs)
    return data


def ItemFactory(**kwargs):
    data = dict(
        id=fake.word(),
        name=fake.word(),
        description=fake.word(),
        created_at=datetime.now().isoformat(),
        updated_at=datetime.now().isoformat(),
        state="completed",
        payment_type_id=2,
        status=22500,
        amount=fake.pyint(),
        deposit_reference=fake.word(),
        buyer_name=fake.last_name(),
        buyer_country="AUS",
        buyer_email=fake.email(),
        seller_name=fake.last_name(),
        seller_country="AUS",
        seller_email=fake.email(),
        tds_check_state="NA",
        currency="AUD",
    )
    data.update(kwargs)
    return data


def WebhookFactory(**kwargs):
    data = dict(
        uuid=fake.word(),
        object_type="transactions",
        http_method="post",
        url=fake.url(),
        description="webhook for transactions",
        enabled=True,
        created_at=datetime.now().isoformat(),
        updated_at=datetime.now().isoformat(),
    )
    data.update(kwargs)
    return data


def DirectDebitAuthorityFactory(**kwargs):
    data = dict(
        id=fake.word(),
        created_at=datetime.now().isoformat(),
        updated_at=datetime.now().isoformat(),
        amount=fake.pyint(),
        bank_bsb=f"{fake.word()},{fake.word()}",
        debit_user_id=f"{fake.word()},{fake.word()}",
        state="approved",
    )
    data.update(kwargs)
    return data




from assembly_payments.services.base import BaseService
from assembly_payments.types import Disbursement, BankAccount, Item, Transaction, User, WalletAccount


class DisbursementService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/disbursements"

    def get(self, disbursement_id):
        response = self._execute(DisbursementService.GET, f"{self.endpoint}/{disbursement_id}")
        return Disbursement(**response['disbursements'])

    def list(self, *args, **kwargs):
        response = self._execute(DisbursementService.GET, self.endpoint)
        return [Disbursement(**disbursement) for disbursement in response.get('disbursements', [])]

    def get_user(self, disbursement_id):
        response = self._execute(DisbursementService.GET, f"{self.endpoint}/{disbursement_id}/users")
        return User(**response['users'])

    def get_wallet_account(self, disbursement_id):
        response = self._execute(DisbursementService.GET, f"{self.endpoint}/{disbursement_id}/wallet_accounts")
        return WalletAccount(**response['wallet_accounts'])

    def get_bank_account(self, disbursement_id):
        response = self._execute(DisbursementService.GET, f"{self.endpoint}/{disbursement_id}/bank_accounts")
        return BankAccount(**response['bank_accounts'])

    def get_items(self, disbursement_id):
        response = self._execute(DisbursementService.GET, f"{self.endpoint}/{disbursement_id}/items")
        return [Item(**item) for item in response.get("items", [])]

    def get_transactions(self, disbursement_id):
        response = self._execute(DisbursementService.GET, f"{self.endpoint}/{disbursement_id}/transactions")
        return [Transaction(**transaction) for transaction in response.get("transactions", [])]



from assembly_payments.services.base import BaseService
from assembly_payments.types import Item, User, ItemRequest, MakePaymentRequest, RefundPaymentRequest, \
    AuthorizePaymentRequest


class ItemService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/items"

    def get(self, item_id):
        response = self._execute(ItemService.GET, f"{self.endpoint}/{item_id}")
        return Item(**response['items'])

    def list(self, *args, **kwargs):
        response = self._execute(ItemService.GET, self.endpoint)
        return [Item(**item) for item in response.get('items', [])]

    def get_seller(self, item_id):
        response = self._execute(ItemService.GET, f"{self.endpoint}/{item_id}/sellers")
        return User(**response['users'])

    def get_buyer(self, item_id):
        response = self._execute(ItemService.GET, f"{self.endpoint}/{item_id}/buyers")
        return User(**response['users'])

    def create(self, **kwargs):
        data = ItemRequest(**kwargs)
        response = self._execute(ItemService.POST, self.endpoint, data=data.dict())
        return Item(**response['items'])

    def make_payment(self, item_id, **kwargs):
        data = MakePaymentRequest(**kwargs)
        response = self._execute(ItemService.PATCH, f"{self.endpoint}/{item_id}/make_payment", data=data.dict())
        return Item(**response['items'])

    def cancel_payment(self, item_id):
        response = self._execute(ItemService.PATCH, f"{self.endpoint}/{item_id}/cancel")
        return Item(**response['items'])

    def refund_payment(self, item_id, **kwargs):
        data = RefundPaymentRequest(**kwargs)
        response = self._execute(ItemService.PATCH, f"{self.endpoint}/{item_id}/refund", data=data.dict(exclude_unset=True))
        return Item(**response['items'])

    def authorize_payment(self, item_id, **kwargs):
        data = AuthorizePaymentRequest(**kwargs)
        response = self._execute(ItemService.PATCH, f"{self.endpoint}/{item_id}/authorize_payment", data=data.dict(exclude_unset=True))
        return Item(**response['items'])

    def capture_payment(self, item_id):
        response = self._execute(ItemService.PATCH, f"{self.endpoint}/{item_id}/capture_payment")
        return Item(**response['items'])

    def void_payment(self, item_id):
        response = self._execute(ItemService.PATCH, f"{self.endpoint}/{item_id}/void_payment")
        return Item(**response['items'])




class PyAssemblyPaymentsException(Exception):
    pass


class PyAssemblyPaymentsNotImplementedException(PyAssemblyPaymentsException):
    pass


class PyAssemblyPaymentsBadRequest(PyAssemblyPaymentsException):
    pass


class PyAssemblyPaymentsForbidden(PyAssemblyPaymentsException):
    pass


class PyAssemblyPaymentsNotFound(PyAssemblyPaymentsException):
    pass


class PyAssemblyPaymentsConflict(PyAssemblyPaymentsException):
    pass


class PyAssemblyPaymentsUnprocessableEntity(PyAssemblyPaymentsException):
    pass